package edu.vub.nfc;

import java.util.List;

import android.app.ActionBar;
import android.app.Activity;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.nfc.NdefMessage;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.tech.Ndef;
import android.nfc.tech.NdefFormatable;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;
import edu.vub.AndroidNFC.R;

public abstract class AbstractWifiJoinerActivity extends Activity {
	
	protected WifiConfig wifiConfig_;
	
	private ActionBar aBar_;
	
	private String tempSsid_ = "";
	private String tempKey_ = "";
	
	private CheckBox useCurrent_;
	protected EditText ssidField_;
	protected EditText keyField_;
	private Button button_;
	
	private ProgressBar spinner_;
	private TextView info_;
	
	private WifiManager wifiManager_;

	protected NfcAdapter mNfcAdapter_;
	
	
	protected abstract int getContentView();
	
	protected abstract String getActivityTitle();
	
	protected abstract String getButtonText();
	

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(getContentView());
		
		wifiManager_ = (WifiManager) this.getSystemService(Context.WIFI_SERVICE);
		
		aBar_ = this.getActionBar();
		aBar_.setTitle(getActivityTitle());
		useCurrent_ = (CheckBox) findViewById(R.id.use_connected);
		ssidField_ = (EditText) findViewById(R.id.ssid_field);
		keyField_ = (EditText) findViewById(R.id.key_field);
		button_ = (Button) findViewById(R.id.save_button);
		spinner_ = (ProgressBar) findViewById(R.id.progressBar1);
		info_ = (TextView) findViewById(R.id.textView2);
		
		mNfcAdapter_ = NfcAdapter.getDefaultAdapter(this);
		
		button_.setText(getButtonText());
		final AbstractWifiJoinerActivity theActivity = this;
		button_.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				String ssid = ssidField_.getText().toString();
				String key = keyField_.getText().toString();
				if (ssid.isEmpty() && key.isEmpty()) {
					toast("You forgot to enter the SSID and key!");
				} else {
					WifiConfig current = new WifiConfig(ssid, key);
					reactOnButtonClick(current);
				}
			}
		});
		
		hideSpinner();
	}
	

	@Override
	protected void onResume() {
		super.onResume();
		Intent i = getIntent();
		i.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP | Intent.FLAG_ACTIVITY_CLEAR_TOP);
		if (i.hasExtra(RawSwipeAndConnectActivity.EXTRA_SSID) && i.hasExtra(RawSwipeAndConnectActivity.EXTRA_KEY)) {
	        ssidField_.setText(i.getStringExtra(RawSwipeAndConnectActivity.EXTRA_SSID));
	        keyField_.setText(i.getStringExtra(RawSwipeAndConnectActivity.EXTRA_KEY));
		} else {
			
			mNfcAdapter_ = NfcAdapter.getDefaultAdapter(this);
			PendingIntent mPendingIntent = PendingIntent.getActivity(this, 0, i, 0);
			
	        IntentFilter ndef = new IntentFilter(NfcAdapter.ACTION_NDEF_DISCOVERED);
	        IntentFilter all = new IntentFilter(NfcAdapter.ACTION_TAG_DISCOVERED);

	        IntentFilter[] mFilters = new IntentFilter[] { ndef, all };
	        String[][] mTechLists = new String[][] { new String[] { Ndef.class.getName() }, new String[] { NdefFormatable.class.getName() } };
			
			mNfcAdapter_.enableForegroundDispatch(this, mPendingIntent, mFilters, mTechLists);
			
			processTagIntent(i);
		}
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		NfcAdapter.getDefaultAdapter(this).disableForegroundDispatch(this);
	}
	
	public void onNewIntent(Intent intent) {
	    Tag tagFromIntent = intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
	    processTagIntent(intent);
	}

	private void processTagIntent(Intent intent) {
		if (NfcAdapter.ACTION_NDEF_DISCOVERED.equals(intent.getAction()) || NfcAdapter.ACTION_TAG_DISCOVERED.equals(intent.getAction()) || NfcAdapter.ACTION_TECH_DISCOVERED.equals(intent.getAction())) {
			// get tag and tag id
			Tag tag = intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
			byte[] tagId = tag.getId();
			// get ndef messages
	        Parcelable[] rawMsgs = intent.getParcelableArrayExtra(NfcAdapter.EXTRA_NDEF_MESSAGES);
	        if (rawMsgs != null) {
	        	NdefMessage ndefMessage;
	            NdefMessage[] msgs = new NdefMessage[rawMsgs.length];
	            for (int i = 0; i < rawMsgs.length; i++) {
	                ndefMessage = (NdefMessage) rawMsgs[i];
	                msgs[i] = ndefMessage;
	                performAction(ndefMessage, tag);
	            }
	        }
	    }
	}
	
	protected abstract void performAction(NdefMessage msg, Tag tag);
	
	// UI helpers
	
	public void onCheckboxClicked(View v) {
	    // Perform action on clicks, depending on whether it's now checked
	    if (useCurrent_.isChecked()) {
	    	WifiConfig current = getCurrentWifiConfig();
	    	if (current == null) {
	    		useCurrent_.setChecked(false);
	    		toast("Cannot retrieve current network information.");
	    	} else {
	    		tempSsid_ = ssidField_.getText().toString();
	    		tempKey_ = keyField_.getText().toString();
		        ssidField_.setText(current.ssid_);
		        //keyField_.setText(current.key_); // XXX cannot retrieve key
		        //disableFields();
		        ssidField_.setEnabled(false);
		        keyField_.requestFocus();
	    	}
	    } else {
	    	ssidField_.setText(tempSsid_);
	    	keyField_.setText(tempKey_);
	    	enableFields();
	    }
	}
	
	private void disableFields() {
		ssidField_.setEnabled(false);
		keyField_.setEnabled(false);
	}
	
	private void enableFields() {
		ssidField_.setEnabled(true);
		keyField_.setEnabled(true);
	}
	
	protected void hideSpinner() {
		spinner_.setVisibility(View.INVISIBLE);
		info_.setVisibility(View.INVISIBLE);
	}
	
	protected void showSpinner() {
		spinner_.setVisibility(View.VISIBLE);
		info_.setVisibility(View.VISIBLE);
	}
	
	protected void setInfo(String info) {
		info_.setText(info);
	}
	
	protected void lock() {
		useCurrent_.setEnabled(false);
		disableFields();
		button_.setEnabled(false);
	}
	
	protected void unlock() {
		useCurrent_.setEnabled(true);
		if (!useCurrent_.isChecked()) {
			enableFields();
		}
		button_.setEnabled(true);
	}
	
	protected void toast(String info) {
		Toast.makeText(this, info, Toast.LENGTH_SHORT).show();
	}
	
	// Wifi
	
	public WifiConfig getCurrentWifiConfig() {
		if (!wifiManager_.isWifiEnabled()) {
			return null;
		}
		WifiInfo wifiInfo = wifiManager_.getConnectionInfo();
		List<WifiConfiguration> configs = wifiManager_.getConfiguredNetworks();
		WifiConfiguration current = null;
		for (WifiConfiguration wc : configs) {
			if (wc.SSID.equals(wifiInfo.getSSID()) || wc.SSID.equals("\"" + wifiInfo.getSSID() + "\"")) {
				current = wc;
				break;
			}
		}
		if (current == null) {
			return null;
		}
		return new WifiConfig(current.SSID.replace("\"", ""), current.preSharedKey);
	}

	
	protected abstract void reactOnButtonClick(WifiConfig wc);
	
	public abstract void writeSucceeded(WifiConfig wc);
	
	public abstract void writeFailed(WifiConfig wc);
	
}

