package edu.vub.nfc;

import android.nfc.NdefMessage;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.tech.Ndef;
import android.nfc.tech.NdefFormatable;
import android.nfc.tech.TagTechnology;
import android.os.Bundle;
import android.util.Log;
import edu.vub.AndroidNFC.R;

public class CreateWifiJoinerActivity extends AbstractWifiJoinerActivity {
	
	private static final int STATE_NORMAL = 0;
	private static final int STATE_WAITING_FOR_TAG = 1;
		
	private int state_ = STATE_NORMAL;
	private Tag pendingTag_ = null;
	private WifiConfig pendingWifiConfig_ = null;
	
	@Override
	protected int getContentView() {
		return R.layout.create;
	}

	@Override
	protected String getActivityTitle() {
		return "Create WiFi joiner";
	}

	@Override
	protected String getButtonText() {
		return "Create";
	}

	@Override
	protected void reactOnButtonClick(WifiConfig wc) {
		state_ = STATE_WAITING_FOR_TAG;
		storeIfStateOk(wc);
	}
	
	@Override
	public void writeSucceeded(WifiConfig wc) {
		hideSpinner();
		unlock();
		clearTags();
		toast("WiFi joiner created!");
	}
	
	@Override
	public void writeFailed(WifiConfig wc) {
		hideSpinner();
		unlock();
		clearTags();
		toast("Creating WiFi joiner failed, try again.");
	}
	
	private void storeIfStateOk(final WifiConfig wc) {
		if (state_ != STATE_WAITING_FOR_TAG) {
			return;
		}
		setInfo("Creating...");
		lock();
		showSpinner();
		
		final NdefMessage msg = wc.createWifiNdefMessage();
		
		if (pendingTag_ != null) {
			final Ndef ndef = Ndef.get(pendingTag_);
			final AbstractWifiJoinerActivity owner = this;
			if (ndef == null) {
				final NdefFormatable ndefFormatable = NdefFormatable.get(pendingTag_);
				new Thread(new Runnable() {

					@Override
					public void run() {
						try {
							ndefFormatable.connect();
							ndefFormatable.format(msg);
							ndefFormatable.close();
							owner.runOnUiThread(new Runnable() {
								public void run() {
									owner.writeSucceeded(wc);
								}
							});
						} catch(Exception e) {
							owner.runOnUiThread(new Runnable() {
								public void run() {
									owner.writeFailed(wc);
								}
							});
						}
					}
					
				}).run();
			} else {
				if (ndef.getMaxSize() < msg.toByteArray().length) {
					Log.e("MORENA", "Data too large to fit in tag memory!");
					this.writeFailed(wc);
				} else {
					new Thread(new Runnable() {

						@Override
						public void run() {
							try {
								ndef.connect();
								ndef.writeNdefMessage(msg);
								ndef.close();
								owner.runOnUiThread(new Runnable() {
									public void run() {
										owner.writeSucceeded(wc);
									}
								});
							} catch(Exception e) {
								owner.runOnUiThread(new Runnable() {
									public void run() {
										owner.writeFailed(wc);
									}
								});
							}
						}
						
					}).run();
					
				};
			}
		} else {
			setInfo("Waiting for tag...");
			state_ = STATE_WAITING_FOR_TAG;
			pendingWifiConfig_  = wc;
		}
	}
	
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setInfo("Waiting for tag...");
	}


	protected void performAction(NdefMessage msg, Tag tag) {
		TagTechnology tagTech = Ndef.get(tag);
		NdefFormatable ndefFormatable = NdefFormatable.get(tag);
		if (tagTech != null || ndefFormatable != null) {
			if (pendingWifiConfig_ != null) {
				pendingTag_ = tag;
				storeIfStateOk(pendingWifiConfig_);
			}
			toast("Found Ndef tag");
		} else {
			Log.i("WifiJoiner", "Action: Not an Ndef tagTech, take no action");
		}
		//this.finish();
	}
	
	
	private void clearTags() {
		pendingWifiConfig_ = null;
		pendingTag_ = null;
	}
}
