package edu.vub.nfc;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.net.wifi.WifiManager;
import android.nfc.NdefMessage;
import android.nfc.NdefRecord;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.tech.Ndef;
import android.nfc.tech.TagTechnology;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.ListView;
import android.widget.Toast;
import edu.vub.AndroidNFC.R;


public class RawSwipeAndConnectActivity extends Activity {
	
	public static final String EXTRA_SSID = "wificonfig_ssid";
	public static final String EXTRA_KEY = "wificonfig_key";
	
	public static RawSwipeAndConnectActivity MAIN = null;
	
	public static WifiConfig toCreate = null;
	
	private ListView wifiJoinerList_;
	
	private WifiManager wifiManager_;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		
		this.getActionBar();

		 
		wifiJoinerList_ = (ListView) findViewById(R.id.listView1);
		wifiJoinerList_.setAdapter(new ConfigListAdapter(this));
		wifiJoinerList_.setTextFilterEnabled(true);
		this.registerForContextMenu(wifiJoinerList_);
		wifiManager_ = (WifiManager) this.getSystemService(Context.WIFI_SERVICE);
				
	}
	
	
	@Override
	protected void onResume() {
		super.onResume();
		Log.d("BLA", "IM IN MAIN " + this.getClass());
		MAIN = this;
		Intent intent = getIntent();
		processTagIntent(intent);
	}
	
		
    // DB helpers



	public SQLiteDatabase getWritableConfigDB() { 
		WifiConfigDb db = new WifiConfigDb(this);
		return db.getWritableDatabase();
	}

	public SQLiteDatabase getReadableConfigDB() { 
		WifiConfigDb db = new WifiConfigDb(this);
		return db.getReadableDatabase();
	}
	
	// NFC
	
	private void processTagIntent(Intent intent) {
		if (NfcAdapter.ACTION_NDEF_DISCOVERED.equals(intent.getAction())) {
			// get tag and tag id
			Tag tag = intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
			byte[] tagId = tag.getId();
			// get ndef messages
	        Parcelable[] rawMsgs = intent.getParcelableArrayExtra(NfcAdapter.EXTRA_NDEF_MESSAGES);
	        if (rawMsgs != null) {
	        	NdefMessage ndefMessage;
	            NdefMessage[] msgs = new NdefMessage[rawMsgs.length];
	            for (int i = 0; i < rawMsgs.length; i++) {
	                ndefMessage = (NdefMessage) rawMsgs[i];
	                msgs[i] = ndefMessage;
	                performAction(ndefMessage, tag);
	            }
	        }
	    }
	}

	private void performAction(NdefMessage msg, Tag tag) {
		TagTechnology tagTech = Ndef.get(tag);
		if (tagTech != null) {
			if (WifiConfig.isWifiJoiner(msg)) {
				NdefRecord[] records = msg.getRecords();
				String ssid = new String(records[1].getPayload());
				String pwd = new String(records[2].getPayload());
				WifiConfig config = new WifiConfig(ssid, pwd);
				((ConfigListAdapter) wifiJoinerList_.getAdapter()).addConfig(config);
				toast("Found Wi-Fi joiner for " + config.ssid_);
			} else {
				Log.i("WifiJoiner", "Action: Not a known tag type, take no action");
			}
		} else {
			Log.i("WifiJoiner", "Action: Not an Ndef tagTech, take no action");
		}
		//this.finish();
	}
	

	
	// Action bar
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.actionmenu, menu);
	    return true;
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
	    switch (item.getItemId()) {
	        case R.id.menu_create:
	            // app icon in action bar clicked; go home
	            Intent intent = new Intent(this, CreateWifiJoinerActivity.class);
	            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
	            startActivity(intent);
	            return true;
	        case R.id.menu_share:
	            // app icon in action bar clicked; go home
	            Intent beamIntent = new Intent(this, BeamWifiJoinerActivity.class);
	            beamIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
	            startActivity(beamIntent);
	            return true;
	        default:
	            return super.onOptionsItemSelected(item);
	    }
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
	    super.onCreateContextMenu(menu, v, menuInfo);
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.connect, menu);
	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item) {
	    AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
    	WifiConfig c = (WifiConfig) wifiJoinerList_.getItemAtPosition(info.position);
	    switch (item.getItemId()) {
	        case R.id.connect:
	            c.connect(wifiManager_);
	            //toast("Connect to " + c.ssid_);
	            return true;
	        case R.id.create:
	        	Intent intent = new Intent(this, CreateWifiJoinerActivity.class);
	        	intent.putExtra(EXTRA_SSID, c.ssid_);
	        	intent.putExtra(EXTRA_KEY, c.key_);
	            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
	            startActivity(intent);
	            return true;
	        case R.id.share:
	        	Intent shareIntent = new Intent(this, BeamWifiJoinerActivity.class);
	        	shareIntent.putExtra(EXTRA_SSID, c.ssid_);
	        	shareIntent.putExtra(EXTRA_KEY, c.key_);
	        	shareIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
	            startActivity(shareIntent);
	            return true;
	        case R.id.remove:
	        	toast("Removed " + c.ssid_);
	        	((ConfigListAdapter) wifiJoinerList_.getAdapter()).removeAt(info.position);
	            return true;
	        default:
	            return super.onContextItemSelected(item);
	    }
	}
	
	private void toast(String info) {
		Toast.makeText(this, info, Toast.LENGTH_SHORT).show();
	}
	
	
}