package edu.vub.nfc;

import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.ListView;
import android.widget.Toast;
import edu.vub.AndroidNFC.R;
import edu.vub.nfc.thing.EmptyRecord;
import edu.vub.nfc.thing.Thing;
import edu.vub.nfc.thing.ThingActivity;


public class SwipeAndConnectActivity extends ThingActivity<WifiConfig> {
	
	public static final String EXTRA_SSID = "wificonfig_ssid";
	public static final String EXTRA_KEY = "wificonfig_key";
	
	public static SwipeAndConnectActivity MAIN = null;
	
	public static WifiConfig toCreate = null;
	
	private ListView wifiJoinerList_;
	
	private WifiManager wifiManager_;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		
		this.getActionBar();

		 
		wifiJoinerList_ = (ListView) findViewById(R.id.listView1);
		wifiJoinerList_.setAdapter(new ConfigListAdapter(this));
		wifiJoinerList_.setTextFilterEnabled(true);
		this.registerForContextMenu(wifiJoinerList_);
		wifiManager_ = (WifiManager) this.getSystemService(Context.WIFI_SERVICE);
				
	}
	
	
	@Override
	protected void onResume() {
		// TODO Auto-generated method stub
		super.onResume();
		MAIN = this;
	}
	
		
    // DB helpers



	public SQLiteDatabase getWritableConfigDB() { 
		WifiConfigDb db = new WifiConfigDb(this);
		return db.getWritableDatabase();
	}

	public SQLiteDatabase getReadableConfigDB() { 
		WifiConfigDb db = new WifiConfigDb(this);
		return db.getReadableDatabase();
	}
	
	// NFC

	@Override
	public void whenDiscovered(WifiConfig c) {
		((ConfigListAdapter) wifiJoinerList_.getAdapter()).addConfig(c);
		toast("Found Wi-Fi joiner for " + c.ssid_);
	}
	
	@Override
	public void whenDiscovered(EmptyRecord r) {
		// do nothing
	}

	@Override
	public Class<? extends Thing> getThingType() {
		return WifiConfig.class;
	}
	
	// Action bar
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.actionmenu, menu);
	    return true;
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
	    switch (item.getItemId()) {
	        case R.id.menu_create:
	            // app icon in action bar clicked; go home
	            Intent intent = new Intent(this, CreateWifiJoinerActivity.class);
	            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
	            startActivity(intent);
	            return true;
	        case R.id.menu_share:
	            // app icon in action bar clicked; go home
	            Intent beamIntent = new Intent(this, BeamWifiJoinerActivity.class);
	            beamIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
	            startActivity(beamIntent);
	            return true;
	        default:
	            return super.onOptionsItemSelected(item);
	    }
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
	    super.onCreateContextMenu(menu, v, menuInfo);
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.connect, menu);
	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item) {
	    AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
    	WifiConfig c = (WifiConfig) wifiJoinerList_.getItemAtPosition(info.position);
	    switch (item.getItemId()) {
	        case R.id.connect:
	            c.connect(wifiManager_);
	            //toast("Connect to " + c.ssid_);
	            return true;
	        case R.id.create:
	        	Intent intent = new Intent(this, CreateWifiJoinerActivity.class);
	        	intent.putExtra(EXTRA_SSID, c.ssid_);
	        	intent.putExtra(EXTRA_KEY, c.key_);
	            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
	            startActivity(intent);
	            return true;
	        case R.id.share:
	        	Intent shareIntent = new Intent(this, BeamWifiJoinerActivity.class);
	        	shareIntent.putExtra(EXTRA_SSID, c.ssid_);
	        	shareIntent.putExtra(EXTRA_KEY, c.key_);
	        	shareIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
	            startActivity(shareIntent);
	            return true;
	        case R.id.remove:
	        	toast("Removed " + c.ssid_);
	        	((ConfigListAdapter) wifiJoinerList_.getAdapter()).removeAt(info.position);
	            return true;
	        default:
	            return super.onContextItemSelected(item);
	    }
	}
	
	private void toast(String info) {
		Toast.makeText(this, info, Toast.LENGTH_SHORT).show();
	}
	
	
}